@@ -1,11 +1,10 @@
 from escnn.kernels.basis import KernelBasis, AdjointBasis, UnionBasis, EmptyBasisException
 from escnn.kernels.steerable_basis import SteerableKernelBasis
-from escnn.kernels.polar_basis import GaussianRadialProfile, SphericalShellsBasis
 from escnn.kernels.wignereckart_solver import WignerEckartBasis, RestrictedWignerEckartBasis
-from escnn.kernels.sparse_basis import SparseSteerableBasis
+from escnn.kernels.sparse_basis import SparseOrbitBasis, SparseOrbitBasisWithIcosahedralSymmetry
 
-from escnn.kernels.spaces import SphereO3, SphereSO3
-from escnn.kernels.spaces import Dodecahedron, Icosahedron, Icosidodecahedron, PointRn
+from escnn.kernels.polar_basis import GaussianRadialProfile
+from escnn.kernels.polar_basis import SphericalShellsBasis
 
 from escnn.group import *
 
@@ -79,15 +78,19 @@ def kernels_SO3_act_R3(in_repr: Representation, out_repr: Representation,
     
     assert isinstance(group, SO3)
 
-    if maximum_frequency is not None:
-        harmonics = [(l,) for l in range(maximum_frequency+1)]
-    else:
-        harmonics = None
-        
-    angular_basis = SteerableKernelBasis(SphereSO3(), in_repr, out_repr, WignerEckartBasis, harmonics=harmonics)
     radial_profile = GaussianRadialProfile(radii, sigma)
 
-    basis = SphericalShellsBasis(3, angular_basis, radial_profile, filter=filter)
+    if maximum_frequency is None:
+        max_in_freq = max(freq for freq, in in_repr.irreps)
+        max_out_freq = max(freq for freq, in out_repr.irreps)
+        maximum_frequency = max_in_freq + max_out_freq
+
+    basis = SteerableKernelBasis(
+        SphericalShellsBasis(maximum_frequency, radial_profile, filter=filter),
+        in_repr, out_repr,
+        RestrictedWignerEckartBasis,
+        sg_id='so3'
+    )
 
     if adjoint is not None and not np.allclose(adjoint, np.eye(2)):
         assert adjoint.shape == (3, 3)
@@ -130,15 +133,18 @@ def kernels_O3_act_R3(in_repr: Representation, out_repr: Representation,
     group = in_repr.group
     assert isinstance(group, O3)
 
-    if maximum_frequency is not None:
-        harmonics = [(k, l) for l in range(maximum_frequency+1) for k in range(2)]
-    else:
-        harmonics = None
-        
-    angular_basis = SteerableKernelBasis(SphereO3(), in_repr, out_repr, WignerEckartBasis, harmonics=harmonics)
     radial_profile = GaussianRadialProfile(radii, sigma)
 
-    basis = SphericalShellsBasis(3, angular_basis, radial_profile, filter=filter)
+    if maximum_frequency is None:
+        max_in_freq = max(freq for _, freq in in_repr.irreps)
+        max_out_freq = max(freq for _, freq in out_repr.irreps)
+        maximum_frequency = max_in_freq + max_out_freq
+
+    basis = SteerableKernelBasis(
+        SphericalShellsBasis(maximum_frequency, radial_profile, filter=filter),
+        in_repr, out_repr,
+        WignerEckartBasis,
+    )
 
     if adjoint is not None and not np.allclose(adjoint, np.eye(2)):
         assert adjoint.shape == (3, 3)
@@ -164,13 +170,15 @@ def kernels_O3_subgroup_act_R3(in_repr: Representation, out_repr: Representation
     assert in_repr.group == group
     assert out_repr.group == group
     
-    harmonics = [(k, l) for l in range(maximum_frequency+1) for k in range(2)]
-    angular_basis = SteerableKernelBasis(SphereO3(), in_repr, out_repr, RestrictedWignerEckartBasis,
-                                         sg_id=sg_id, harmonics=harmonics)
     radial_profile = GaussianRadialProfile(radii, sigma)
-    
-    basis = SphericalShellsBasis(3, angular_basis, radial_profile, filter=filter)
-    
+
+    basis = SteerableKernelBasis(
+        SphericalShellsBasis(maximum_frequency, radial_profile, filter=filter),
+        in_repr, out_repr,
+        RestrictedWignerEckartBasis,
+        sg_id=sg_id
+    )
+
     if adjoint is not None and not np.allclose(adjoint, np.eye(2)):
         assert adjoint.shape == (3, 3)
         basis = AdjointBasis(basis, adjoint)
@@ -192,14 +200,19 @@ def kernels_SO3_subgroup_act_R3(in_repr: Representation, out_repr: Representatio
     group, _, _ = so3.subgroup(sg_id)
     assert in_repr.group == group
     assert out_repr.group == group
+
+    o3 = o3_group(maximum_frequency)
+    sg_id = o3._combine_subgroups('so3', sg_id)
     
-    harmonics = [(l,) for l in range(maximum_frequency+1)]
-    angular_basis = SteerableKernelBasis(SphereSO3(), in_repr, out_repr, RestrictedWignerEckartBasis,
-                                         sg_id=sg_id, harmonics=harmonics)
     radial_profile = GaussianRadialProfile(radii, sigma)
-    
-    basis = SphericalShellsBasis(3, angular_basis, radial_profile, filter=filter)
-    
+
+    basis = SteerableKernelBasis(
+        SphericalShellsBasis(maximum_frequency, radial_profile, filter=filter),
+        in_repr, out_repr,
+        RestrictedWignerEckartBasis,
+        sg_id=sg_id
+    )
+
     if adjoint is not None and not np.allclose(adjoint, np.eye(2)):
         assert adjoint.shape == (3, 3)
         basis = AdjointBasis(basis, adjoint)
@@ -259,20 +272,18 @@ def kernels_Tetra_act_R3(in_repr: Representation, out_repr: Representation,
 
 ###### Icosahedral Symmetry with Aliased samples
 
-def kernels_aliased_Ico_act_R3_dodecahedron(in_repr: Representation, out_repr: Representation,
-                                            radii: List[float],
-                                            sigma: Union[List[float], float],
-                                            adjoint: np.ndarray = None,
-                                            ) -> KernelBasis:
+def _kernels_aliased_Ico_act_R3(in_repr: Representation, out_repr: Representation,
+                                radii: List[float],
+                                sigma: Union[List[float], float],
+                                sgid: Tuple,
+                                adjoint: np.ndarray = None,
+                                ) -> KernelBasis:
 
     group = ico_group()
 
     assert in_repr.group == group
     assert out_repr.group == group
 
-    dodecahedron = Dodecahedron()
-    origin = PointRn(3, group)
-
     if isinstance(sigma, float):
         sigma = [sigma]*len(radii)
     assert len(sigma) == len(radii)
@@ -282,11 +293,23 @@ def kernels_aliased_Ico_act_R3_dodecahedron(in_repr: Representation, out_repr: R
         attributes = {'radius': r}
         try:
             if np.isclose(r, 0., rtol=1e-7, atol=1e-7):
-                basis = SparseSteerableBasis(origin, in_repr, out_repr, sigma=s, attributes=attributes)
+                basis = SparseOrbitBasis(
+                    X=group.homspace(group.subgroup_self_id),
+                    action=group.standard_representation,
+                    root=np.zeros(3),
+                    sigma=s, attributes=attributes
+                )
             else:
                 change_of_basis = np.eye(3) * r
-                basis = SparseSteerableBasis(dodecahedron, in_repr, out_repr, sigma=s, change_of_basis=change_of_basis, attributes=attributes)
-            basis_list.append(basis)
+                basis = SparseOrbitBasisWithIcosahedralSymmetry(
+                    X=group.homspace(sgid),
+                    sigma=s, attributes=attributes,
+                    change_of_basis = change_of_basis
+                )
+
+            basis_list.append(
+                SteerableKernelBasis(basis, in_repr, out_repr, WignerEckartBasis)
+            )
         except EmptyBasisException:
             pass
 
@@ -299,45 +322,27 @@ def kernels_aliased_Ico_act_R3_dodecahedron(in_repr: Representation, out_repr: R
     return basis
 
 
+def kernels_aliased_Ico_act_R3_dodecahedron(in_repr: Representation, out_repr: Representation,
+                                            radii: List[float],
+                                            sigma: Union[List[float], float],
+                                            adjoint: np.ndarray = None,
+                                            ) -> KernelBasis:
+
+    sgid = (False, 5)
+    return _kernels_aliased_Ico_act_R3(
+        in_repr, out_repr, radii, sigma, sgid, adjoint
+    )
+
+
 def kernels_aliased_Ico_act_R3_icosahedron(in_repr: Representation, out_repr: Representation,
                                            radii: List[float],
                                            sigma: Union[List[float], float],
                                            adjoint: np.ndarray = None,
                                            ) -> KernelBasis:
-
-    group = ico_group()
-
-    assert in_repr.group == group
-    assert out_repr.group == group
-
-    icosahedron = Icosahedron()
-    origin = PointRn(3, group)
-
-    if isinstance(sigma, float):
-        sigma = [sigma]*len(radii)
-
-    assert len(sigma) == len(radii)
-
-    basis_list = []
-    for r, s in zip(radii, sigma):
-        attributes = {'radius': r}
-        try:
-            if np.isclose(r, 0., rtol=1e-7, atol=1e-7):
-                basis = SparseSteerableBasis(origin, in_repr, out_repr, sigma=s, attributes=attributes)
-            else:
-                change_of_basis = np.eye(3) * r
-                basis = SparseSteerableBasis(icosahedron, in_repr, out_repr, sigma=s, change_of_basis=change_of_basis, attributes=attributes)
-            basis_list.append(basis)
-        except EmptyBasisException:
-            pass
-
-    basis = UnionBasis(basis_list)
-
-    if adjoint is not None and not np.allclose(adjoint, np.eye(2)):
-        assert adjoint.shape == (3, 3)
-        basis = AdjointBasis(basis, adjoint)
-
-    return basis
+    sgid = (False, 3)
+    return _kernels_aliased_Ico_act_R3(
+        in_repr, out_repr, radii, sigma, sgid, adjoint
+    )
 
 
 def kernels_aliased_Ico_act_R3_icosidodecahedron(in_repr: Representation, out_repr: Representation,
@@ -346,39 +351,10 @@ def kernels_aliased_Ico_act_R3_icosidodecahedron(in_repr: Representation, out_re
                                                  adjoint: np.ndarray = None,
                                                  ) -> KernelBasis:
 
-    group = ico_group()
-
-    assert in_repr.group == group
-    assert out_repr.group == group
-
-    icosahedron = Icosidodecahedron()
-    origin = PointRn(3, group)
-
-    if isinstance(sigma, float):
-        sigma = [sigma]*len(radii)
-
-    assert len(sigma) == len(radii)
-
-    basis_list = []
-    for r, s in zip(radii, sigma):
-        attributes = {'radius': r}
-        try:
-            if np.isclose(r, 0., rtol=1e-7, atol=1e-7):
-                basis = SparseSteerableBasis(origin, in_repr, out_repr, sigma=s, attributes=attributes)
-            else:
-                change_of_basis = np.eye(3) * r
-                basis = SparseSteerableBasis(icosahedron, in_repr, out_repr, sigma=s, change_of_basis=change_of_basis, attributes=attributes)
-            basis_list.append(basis)
-        except EmptyBasisException:
-            pass
-
-    basis = UnionBasis(basis_list)
-
-    if adjoint is not None and not np.allclose(adjoint, np.eye(2)):
-        assert adjoint.shape == (3, 3)
-        basis = AdjointBasis(basis, adjoint)
-
-    return basis
+    sgid = (False, 2)
+    return _kernels_aliased_Ico_act_R3(
+        in_repr, out_repr, radii, sigma, sgid, adjoint
+    )
 
 
 ###### Full Platonic Symmetries
@@ -391,6 +367,8 @@ def kernels_FullIco_act_R3(in_repr: Representation, out_repr: Representation,
                            adjoint: np.ndarray = None,
                            filter: Callable[[Dict], bool] = None
                            ) -> KernelBasis:
+    # TODO
+    raise NotImplementedError
     # I_h = I x C_2
     
     sg_id = (True, 'ico')
@@ -409,6 +387,8 @@ def kernels_FullOcta_act_R3(in_repr: Representation, out_repr: Representation,
                             adjoint: np.ndarray = None,
                             filter: Callable[[Dict], bool] = None
                             ) -> KernelBasis:
+    # TODO
+    raise NotImplementedError
     # O_h = O x C_2
     
     sg_id = (True, 'octa')
@@ -427,6 +407,8 @@ def kernels_FullTetra_act_R3(in_repr: Representation, out_repr: Representation,
                              adjoint: np.ndarray = None,
                              filter: Callable[[Dict], bool] = None
                              ) -> KernelBasis:
+    # TODO
+    raise NotImplementedError
     # group T_d
     # n.b. this is different from T x C_2
     
@@ -446,6 +428,8 @@ def kernels_Pyrito_act_R3(in_repr: Representation, out_repr: Representation,
                           adjoint: np.ndarray = None,
                           filter: Callable[[Dict], bool] = None
                           ) -> KernelBasis:
+    # TODO
+    raise NotImplementedError
     # T_h = T x C_2
     # n.b. not a symmetry of the tetrahedron
     
@@ -468,6 +452,7 @@ def kernels_SO2_act_R3(in_repr: Representation, out_repr: Representation,
                        adjoint: np.ndarray = None,
                        filter: Callable[[Dict], bool] = None
                        ) -> KernelBasis:
+
     sg_id = (False, -1)
     return kernels_SO3_subgroup_act_R3(
         in_repr, out_repr, sg_id,
